from enum import Enum
import copy


class MultipartException(Exception):
    pass


class MultipartParser:
    class MultipartState(Enum):
        BEGIN = 0
        BOUNDARY_DEF = 1
        HEADERS = 2
        IMAGE = 3
        DONE = 4

    def __init__(self, in_stream):
        self.in_stream = in_stream
        self.stream_start = in_stream.tell()
        self.boundary = b''
        self.img_headers = {}
        self.image = b''
        self.state = self.MultipartState.BEGIN
        self.reset_buffers()

    def __iter__(self):
        self.in_stream.seek(self.stream_start)
        self.boundary = b''
        self.state = self.MultipartState.BEGIN
        self.reset_buffers()
        return self

    def reset_buffers(self):
        self.img_headers = {}
        self.image = b''

    def get_next_byte(self):
        b = self.in_stream.read(1)
        if b == b'':
            raise EOFError
        return b

    def get_to_boundary_definition(self, byt):
        while byt != b'-':
            byt = self.get_next_byte()
        self.boundary = byt

    def get_boundary_definition(self, byt):
        while byt != b'\r':
            self.boundary += byt
            byt = self.get_next_byte()
        byt = self.get_next_byte()
        if byt != b'\n':
            raise MultipartException

    def get_headers(self, byt):
        buf = b''
        while True:
            if byt == b'\r':
                byt = self.get_next_byte()
                if byt != b'\n':
                    raise MultipartException
                if len(buf) == 0:
                    return
                h = buf.decode('utf-8').split(':', 1)
                self.img_headers[h[0].strip()] = h[1].strip()
                buf = b''
            else:
                buf += byt
            byt = self.get_next_byte()

    def check_for_boundary(self, byt):
        buf = b''
        while True:
            buf += byt
            if not self.boundary.startswith(buf):
                self.image += buf
                return False
            elif self.boundary == buf:
                byt = self.get_next_byte()
                if byt == b'\r':
                    self.get_next_byte();
                    return True;
                elif byt == b'-':
                    byt = self.get_next_byte()
                    if byt == b'-':
                        self.state = self.MultipartState.DONE
                        return True
                    else:
                        self.image += buf + b'-' + byt
                        return False
                else:
                    self.image += buf + byt
                    return False
            byt = self.get_next_byte()

    def emit_data_and_cleanup(self):
        h = copy.deepcopy(self.img_headers)
        i = self.image[:].rstrip(b'\r\n')
        self.reset_buffers()
        self.state = self.MultipartState.HEADERS
        return h, i

    def __next__(self):
        try:
            while True:
                if self.state == self.MultipartState.DONE:
                    raise StopIteration

                b = self.get_next_byte()
                if self.state == self.MultipartState.BEGIN:
                    self.get_to_boundary_definition(b)
                    self.state = self.MultipartState.BOUNDARY_DEF
                elif self.state == self.MultipartState.BOUNDARY_DEF:
                    self.get_boundary_definition(b)
                    self.state = self.MultipartState.HEADERS
                elif self.state == self.MultipartState.HEADERS:
                    self.get_headers(b)
                    self.state = self.MultipartState.IMAGE
                elif self.state == self.MultipartState.IMAGE:
                    if b == b'-':
                        if self.check_for_boundary(b):
                            return self.emit_data_and_cleanup()
                    else:
                        self.image += b
        except EOFError:
            if len(self.image):
                return self.emit_data_and_cleanup()
            else:
                raise StopIteration
